body {
    background: #f0f8ff;
    color: black;
    font-family: monospace;
    font-size: 25px;
    padding-left: 30px;
    padding-right: 30px;
    padding-bottom: 50px;
}

#query{
    font-size: 20px;
    margin: 20px;
    min-height: 25px;
    max-height: 50px;
    width: 100%;
    border-radius: 5px;
    border-color: #7a7a9f;
    border-width: 2px;
    padding: 10px;
    resize: none;
}

pre {
    white-space: pre-wrap; /* css-3 */
    white-space: -moz-pre-wrap; /* Mozilla, since 1999 */
    white-space: -pre-wrap; /* Opera 4-6 */
    white-space: -o-pre-wrap; /* Opera 7 */
    word-wrap: break-word; /* Internet Explorer 5.5+ */
}

pre .dialogue {
    width: 50vw;
    height: 90vh;
    display: flex;
    padding: 10px;
    margin-left: 8px;
    flex-direction: column-reverse;
}

pre .logs {
    width: 35vw;
    height: 90vh;
    display: flex;
    padding: 10px;
    margin-left: 15px;
    flex-direction: column-reverse;
    background: white;
    opacity: 50%;
}

.controls {
    position: fixed;
    bottom: 0;
    width: 100%;
    padding: 10px 10px;
    border-top: 1px solid #4e4a4a;
}

#banner {
    width: 100%;
    padding: 10px 10px;
}

#default-sidebar a{
    cursor: pointer;
    text-align: left;
}

.dialogue-row-user {
    font-family: monospace;
    font-size: 30px;
    margin-top: 20px;
    margin-bottom: 10px;
    padding: 10px;
    width: 80%;
}

.dialogue-row-bot {
    font-family: monospace;
    font-size: 30px;
    color: gray;
    margin-top: 20px;
    margin-bottom: 15px;
    border-radius: 5px;
    border-color: #7a7a9f;
    border-width: 2px;
    background-color: whitesmoke;
    padding: 10px;
}

.log-row {
    font-size: 11px;
    margin-left: 30px;
    margin-top: 10px;
    color: #2a2a2a;
    margin-bottom: 10px;
    padding: 10px;
}


.dialogue-row-comment {
    font-family: monospace;
    font-size: 30px;
    margin-top: 20px;
    margin-bottom: 15px;
    border-radius: 5px;
    border-color: #7a7a9f;
    border-width: 2px;
    padding: 10px;
}

#code {
    font-family: monospace;
    color: #1d9d01;
    background-color: #1f1d1d;
    padding: 10px;
    border-radius: 5px;
    width: 100%;
    resize: none;
}

execute {
    color: green;
}

rememeber {
    color: blue;
}